package plugins

import BuildAndroidConfig
import BuildPlugins
import com.android.build.gradle.AppExtension
import org.gradle.api.JavaVersion
import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.plugins.ExtensionAware
import org.gradle.kotlin.dsl.apply
import org.gradle.plugin.use.PluginDependenciesSpec
import org.gradle.plugin.use.PluginDependencySpec
import org.jetbrains.kotlin.gradle.dsl.KotlinJvmOptions

/**
 * Gradle plugin implemented by [SharedAndroidDynamicModulePlugin].
 *
 * @see [SharedAndroidDynamicModulePlugin]
 */
inline val PluginDependenciesSpec.`shared-android-dynamic-module`: PluginDependencySpec
    get() = id("shared-android-dynamic-module")

class SharedAndroidDynamicModulePlugin : Plugin<Project> {

    override fun apply(target: Project): Unit = target.run {
        project.group = BuildAndroidConfig.GROUP
        project.version = BuildAndroidConfig.VERSION_NAME

        configAndroidDynamicModule()
    }

    private fun Project.configAndroidDynamicModule() = run {
        apply(plugin = BuildPlugins.ANDROID_DYNAMIC_MODULE)
        apply(plugin = BuildPlugins.KOTLIN_ANDROID)

        android {
            compileSdkVersion(BuildAndroidConfig.COMPILE_SDK_VERSION)
            buildToolsVersion(BuildAndroidConfig.BUILD_TOOLS_VERSION)

            defaultConfig {
                minSdkVersion(BuildAndroidConfig.MIN_SDK_VERSION)
                targetSdkVersion(BuildAndroidConfig.TARGET_SDK_VERSION)

                versionCode = BuildAndroidConfig.VERSION_CODE
                versionName = BuildAndroidConfig.VERSION_NAME

                testInstrumentationRunner = BuildAndroidConfig.TEST_INSTRUMENTATION_RUNNER

                consumerProguardFiles("consumer-rules.pro")
            }

            compileOptions {
                sourceCompatibility = JavaVersion.VERSION_1_8
                targetCompatibility = JavaVersion.VERSION_1_8
            }

            kotlinOptions {
                jvmTarget = JavaVersion.VERSION_1_8.toString()
            }

            dataBinding {
                isEnabled = true
            }

            sourceSets {
                getByName("main") {
                    java.srcDir("src/main/java")
                }
                getByName("test") {
                    java.srcDir("src/test/java")
                }
                getByName("androidTest") {
                    java.srcDir("src/androidTest/java")
                }
            }
        }
    }

    private fun Project.android(configure: AppExtension.() -> Unit): Unit =
        (this as ExtensionAware).extensions.configure("android", configure)

    private fun AppExtension.kotlinOptions(configure: KotlinJvmOptions.() -> Unit): Unit =
        (this as ExtensionAware).extensions.configure("kotlinOptions", configure)

}