package vn.teko.sandbox.connector

import android.app.Application
import android.content.Intent
import vn.teko.hestia.android.model.AppLauncherData
import vn.teko.hestia.android.nativelib.AndroidNativeAppLauncher
import vn.teko.hestia.core.model.authentication.idtoken.IdTokenExchangeAuthResult
import vn.teko.sandbox.nativetest.ui.MiniAppShoppingActivity


/**
 * This is where you can config to open your first mini-app screen
 */
internal class NativeConnector : AndroidNativeAppLauncher {

    /**
     * @param application: Application that was used to open new activity
     * @param launcherData: Containing exchanged token and some extra data that you want to dynamic
     * config your mini-app
     * @return Intent
     */
    override suspend fun create(application: Application, launcherData: AppLauncherData): Intent {

        /**
         * Reconfigure the intent to open your activity here
         * For example: return Intent(application, AwesomeActivity::class.java)
         */
        return Intent(application, MiniAppShoppingActivity::class.java).apply {
            val authResult = launcherData.authResult as? IdTokenExchangeAuthResult
            authResult?.idToken?.let {
                putExtra(ID_TOKEN_KEY, it)
            }

            launcherData.extraConfig.forEach {
                putExtra(it.key, it.value)
            }
        }
    }

    companion object {
        private const val ID_TOKEN_KEY = "idToken"
    }
}