package vn.teko.sandbox.ui.home

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.map
import vn.teko.android.core.ui.base.BaseView
import vn.teko.android.core.ui.base.BaseViewModel
import vn.teko.android.core.ui.util.ViewState
import vn.teko.android.core.util.android.livedata.SingleHandlerEvent
import vn.teko.android.terra.auth.TerraAuthInterface
import vn.teko.hestia.core.HestiaCallback
import vn.teko.hestia.core.HestiaInterface
import vn.teko.hestia.core.model.AppType
import vn.teko.hestia.core.util.HestiaError
import vn.teko.sandbox.ui.home.model.SSA_APP_CODE_SCENARIO_3
import vn.teko.sandbox.ui.home.model.Scenario
import javax.inject.Inject


/**
 * Created by TrungCS on 26/09/2021.
 * Email: trung.cs@teko.vn
 * Company: Teko
 */
class HomeViewModel @Inject constructor(
    private val terraAuthInterface: TerraAuthInterface,
    private val hestiaInterface: HestiaInterface,
) : BaseViewModel<BaseView>() {

    val miniAppCode = MutableLiveData(SSA_APP_CODE_SCENARIO_3)

    private val _openLoginEvent = MutableLiveData<SingleHandlerEvent<Unit>>()
    val openLoginEvent: LiveData<SingleHandlerEvent<Unit>>
        get() = _openLoginEvent

    private val _openMiniAppErrorEvent = MutableLiveData<SingleHandlerEvent<HestiaError>>()
    val openMiniAppErrorEvent: LiveData<SingleHandlerEvent<HestiaError>>
        get() = _openMiniAppErrorEvent

    private val _logoutEvent = MutableLiveData<SingleHandlerEvent<Unit>>()
    val logoutEvent: LiveData<SingleHandlerEvent<Unit>>
        get() = _logoutEvent

    private val _currentMiniAppType = MutableLiveData(AppType.ReactNativeAndroid)
    val currentMiniAppType: LiveData<AppType> = _currentMiniAppType

    private val _currentScenario = MutableLiveData(Scenario.Default)
    val currentScenario: LiveData<Scenario> = _currentScenario

    private val _isLogged = MutableLiveData<Boolean>()
    val isLogged: LiveData<Boolean> = _isLogged

    val showLoginButton: LiveData<Boolean> = _currentScenario.map {
        !it.isDefault()
    }

    val showMiniAppCodeInput: LiveData<Boolean> = _currentScenario.map {
        it.isPayment()
    }

    fun updateLoginStatus() {
        terraAuthInterface.isAuthorized {
            _isLogged.value = it
        }
    }

    fun setScenario(scenario: Scenario) {
        _currentScenario.value = scenario
    }

    fun setMiniAppType(appType: AppType) {
        _currentMiniAppType.value = appType
    }

    fun handleLogin() {
        if (_isLogged.value == true) {
            _logoutEvent.value = SingleHandlerEvent(Unit)
        } else {
            _openLoginEvent.value = SingleHandlerEvent(Unit)
        }
    }

    fun openMiniApp() {
        setState(ViewState.LOADING)
        val callback = object : HestiaCallback {
            override fun onError(error: HestiaError) {
                setState(ViewState.IDLE)
                _openMiniAppErrorEvent.value = SingleHandlerEvent(error)
            }

            override fun onSuccess() {
                setState(ViewState.IDLE)
            }
        }

        val currentMiniAppCode = when (val scenario = getScenario()) {
            Scenario.Payment -> miniAppCode.value ?: ""
            else -> scenario.miniAppCode
        }

        hestiaInterface.startApp(
            currentMiniAppCode,
            getMiniAppType(),
            mapOf(),
            callback
        )
    }

    private fun getMiniAppType() = currentMiniAppType.value ?: AppType.NativeAndroid

    private fun getScenario() = currentScenario.value ?: Scenario.Default
}

